//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "FilesAppDelegate.h"

#import "U1AccountManager.h"
#import "U1DataRepository.h"
#import "U1FileNode.h"
#import "U1FilePreviewViewController.h"
#import "U1FilesClient.h"
#import "U1FilesService.h"
#import "U1FolderNode.h"
#import "U1FolderViewController.h"
#import "U1LoginController.h"
#import "U1Node.h"
#import "U1VolumesViewController.h"


@interface FilesAppDelegate () <U1FolderViewControllerDelegate, U1VolumesViewControllerDelegate, U1LoginControllerDelegate>
@property (retain) U1FilesService *filesService;
@property (retain) U1FilesClient *filesClient;
@property (retain) U1LoginController *loginController;
@property (retain) U1DataRepository *dataRepository;
- (NSURL*)dataStoreURL;
- (void)showVolumes;
- (void)showLoginController;
@end


@implementation FilesAppDelegate

@synthesize window;
@synthesize filesService, filesClient, navController, loginController, dataRepository;

- (void)dealloc
{
	[window release];
	[filesService release];
	[filesClient release];
	[navController release];
	[loginController release];
	[dataRepository release];
    [super dealloc];
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
	[self.window makeKeyAndVisible];

	if (![[U1AccountManager sharedAccountManager] hasCredentials])
	{
		[self showLoginController];
	}
	else
	{
		[self showVolumes];
	}

	return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application;
{
	[self.dataRepository save:NULL];
}

- (void)applicationWillTerminate:(UIApplication *)application;
{
	// TODO: check error
	[self.dataRepository save:NULL];
}

- (NSURL*)dataStoreURL;
{
	NSURL *cachesDirectory = [[[NSFileManager defaultManager] URLsForDirectory:NSCachesDirectory inDomains:NSUserDomainMask] lastObject];
	NSURL *storeURL = [NSURL URLWithString:@"U1Files.sqlite" relativeToURL:cachesDirectory];
	return storeURL;
}

- (void)showVolumes;
{
	filesService = [U1FilesService new];
	
	NSURL *storeURL = [self dataStoreURL];
	self.dataRepository = [[U1DataRepository alloc] initWithStoreURL:storeURL];
	[dataRepository release];

	filesClient = [[U1FilesClient alloc] init];
	[self.filesClient setFilesService:filesService];
	[self.filesClient setDataRepository:self.dataRepository];

	U1VolumesViewController *rootController = [U1VolumesViewController new];
	rootController.delegate = self;
	rootController.filesService = self.filesService;
	
	[self.navController setViewControllers:[NSArray arrayWithObject:rootController] animated:NO];
}

- (void)showLoginController;
{
	self.loginController = [[[U1LoginController alloc] init] autorelease];
	self.loginController.delegate = self;
	[self.navController presentModalViewController:self.loginController.loginViewController animated:YES];
}

- (void)loginControllerDidLogin;
{
	[self.navController dismissModalViewControllerAnimated:YES];
	self.loginController = nil;
	[self showVolumes];
}


#pragma mark U1FolderViewControllerDelegate Methods

- (void)folderController:(U1FolderViewController*)folderController didSelectNode:(U1Node*)node;
{
	if ([node isFolder])
	{
		U1FolderViewController *nextFolderController = [[U1FolderViewController alloc] initWithNodeResourcePath:node.resourcePath];
		nextFolderController.filesClient = self.filesClient;
		nextFolderController.delegate = self;
		[self.navController pushViewController:nextFolderController animated:YES];
	}
	else if ([node isFile])
	{
		U1FilePreviewViewController *fileController = [[U1FilePreviewViewController alloc] initWithNode:(id)node];
		fileController.filesClient = self.filesClient;
		[self.navController pushViewController:fileController animated:YES];
	}
}


#pragma mark U1VolumesControllerDelegate Methods

- (void)volumesController:(U1VolumesViewController *)volumesController didSelectVolume:(NSDictionary *)volume;
{
	NSString *nodePath = [volume objectForKey:@"node_path"];
	U1FolderViewController *folderController = [[U1FolderViewController alloc] initWithNodeResourcePath:nodePath];
	folderController.filesClient = self.filesClient;
	folderController.delegate = self;
	[self.navController pushViewController:folderController animated:YES];
}

@end
