//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1FolderViewController.h"

#import "U1ByteSizeValueTransformer.h"
#import "U1FileNode.h"
#import "U1FilesClient.h"
#import "U1FolderItemCell.h"
#import "U1FolderNode.h"
#import "U1Node.h"
#import "UIActionSheet+U1Additions.h"


@interface U1FolderViewController () <UINavigationControllerDelegate, UIImagePickerControllerDelegate, NSFetchedResultsControllerDelegate>
@property (copy) NSString *resourcePath;
@property (retain) U1ByteSizeValueTransformer *byteSizeTransformer;
@property (retain) NSIndexPath *pendingIndexPath;
@property (retain) U1FolderNode *node;
@property (retain) NSMutableArray *nodes;
@property (retain) NSFetchedResultsController *resultsController;
- (void)loadInfoForNode;
- (void)deleteNodeAtIndexPath:(NSIndexPath*)indexPath;
@end


@implementation U1FolderViewController

@synthesize loadingCell;
@synthesize delegate, filesClient, node, resourcePath;
@synthesize byteSizeTransformer, pendingIndexPath, nodes, resultsController;

- (id)initWithNodeResourcePath:(NSString*)theResourcePath;
{
	self = [super initWithNibName:@"U1FolderViewController" bundle:nil];
	if (!self)
		return nil;
	self.title = [theResourcePath lastPathComponent];
	self.resourcePath = [theResourcePath copy];
	self.byteSizeTransformer = [[U1ByteSizeValueTransformer new] autorelease];
	
	UIBarButtonItem *uploadItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(uploadImage:)];
	[self.navigationItem setRightBarButtonItem:uploadItem];
	
	UILabel *titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, 100, 20)];
	// TODO: Ubuntu font
	[titleLabel setFont:[UIFont boldSystemFontOfSize:18.f]];
	[titleLabel setTextColor:[UIColor blackColor]];
	[titleLabel setShadowColor:[UIColor whiteColor]];
	[titleLabel setShadowOffset:CGSizeMake(-1, -1)];
	[titleLabel setText:self.title];
	[titleLabel sizeToFit];
	[titleLabel setOpaque:NO];
	[titleLabel setBackgroundColor:[UIColor clearColor]];
	[self.navigationItem setTitleView:titleLabel];
	[titleLabel release];
	[uploadItem release];
	
	return self;
}

- (void)dealloc;
{
	[loadingCell release];
	[filesClient release];
	[resourcePath release];
	[node release];
	[byteSizeTransformer release];
	[pendingIndexPath release];
	[nodes release];
	[resultsController release];
	[super dealloc];
}

- (void)viewDidLoad;
{
	[super viewDidLoad];
	[self refresh];
	
	UILongPressGestureRecognizer *longPress = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(longPress:)];
	[longPress setMinimumPressDuration:1.f];
	[self.tableView addGestureRecognizer:longPress];
	[longPress release];
}

- (void)viewDidUnload;
{
	[loadingCell release], loadingCell = nil;
	[super viewDidUnload];
}

- (void)viewDidAppear:(BOOL)animated;
{
	[super viewDidAppear:animated];
	[self.tableView deselectRowAtIndexPath:[self.tableView indexPathForSelectedRow] animated:YES];
}

- (void)longPress:(UILongPressGestureRecognizer*)longPress;
{
	if (longPress.state == UIGestureRecognizerStateBegan)
	{
		CGPoint p = [longPress locationInView:self.tableView];
		NSIndexPath *indexPath = [self.tableView indexPathForRowAtPoint:p];
		U1Node *childNode = [self.nodes objectAtIndex:indexPath.row];

		UIActionSheet *actionSheet = [UIActionSheet new];
		
		[actionSheet setTitle:[[childNode path] lastPathComponent]];
		[actionSheet setDestructiveButtonIndex:[actionSheet addButtonWithTitle:@"Delete"]];
		// TODO: rename
		NSInteger publishButtonIndex = -1;
		NSInteger copyURLButtonIndex = -1;
		if ([childNode isFile])
		{
			U1FileNode *fileNode = (id)childNode;
			if ([[fileNode isPublic] boolValue])
			{
				publishButtonIndex = [actionSheet addButtonWithTitle:@"Unpublish"];
				copyURLButtonIndex = [actionSheet addButtonWithTitle:@"Copy Public URL"];
			}
			else
			{
				publishButtonIndex = [actionSheet addButtonWithTitle:@"Publish"];
			}
		}
		[actionSheet setCancelButtonIndex:[actionSheet addButtonWithTitle:@"Cancel"]];

		self.pendingIndexPath = indexPath;
		[actionSheet showInView:self.view withCompletionBlock:^(NSInteger buttonIndex) {
			if (buttonIndex == [actionSheet destructiveButtonIndex])
			{
				[self deleteNodeAtIndexPath:self.pendingIndexPath];
			}
			else if (buttonIndex == publishButtonIndex)
			{
				U1FileNode *fileNode = (id)childNode;
				if ([[fileNode isPublic] boolValue])
					[self.filesClient unpublishNode:fileNode completionBlock:^(U1FileNode *updatedNode, NSError *error) {
					}];
				else
					[self.filesClient publishNode:fileNode completionBlock:^(U1FileNode *updatedNode, NSError *error) {
					}];
			}
			else if (buttonIndex == copyURLButtonIndex)
			{
				U1FileNode *fileNode = (id)childNode;
				NSArray *types = UIPasteboardTypeListString;
				[[UIPasteboard generalPasteboard] setValue:[fileNode publicURLString] forPasteboardType:[types lastObject]];
			}
			else if (buttonIndex == [actionSheet cancelButtonIndex])
			{
//				NSLog(@"cancel");
			}
			self.pendingIndexPath = nil;
		}];
	}
}

- (void)uploadImage:(id)sender;
{
	UIImagePickerController *imagePicker = [[UIImagePickerController alloc] init];
	imagePicker.delegate = self;
	[self presentModalViewController:imagePicker animated:YES];
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info;
{
	// TODO: check for jpg/png
	// TODO: handle videos (stream)
	
	[picker dismissModalViewControllerAnimated:YES];
	
//	NSURL *imageURL = [info objectForKey:UIImagePickerControllerReferenceURL];
	UIImage *image = [info objectForKey:UIImagePickerControllerOriginalImage];
	
	// TODO: pull date from ALAsset metadata
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	[dateFormatter setDateStyle:NSDateFormatterMediumStyle];
	[dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
	NSString *filename = [NSString stringWithFormat:@"Photo %@.png", [dateFormatter stringFromDate:[NSDate date]]];

	[self.filesClient uploadContentData:UIImagePNGRepresentation(image) toFolder:self.node withResourceName:filename progressBlock:^(long long bytesUploaded, long long totalBytes) {
		
	} completionBlock:^(U1FileNode *updatedNode, NSError *error) {
		
	}];
}


#pragma mark UITableViewDataSource Methods

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section;
{
	if (!self.resultsController)
		return 1;
	
	return [[[self.resultsController sections] objectAtIndex:section] numberOfObjects];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath;
{
	if (!self.resultsController)
	{
		return self.loadingCell;
	}
	
	U1FolderItemCell *cell = [U1FolderItemCell cellForTableView:tableView];
	
	U1Node *childNode = [self.resultsController objectAtIndexPath:indexPath];
	cell.nameLabel.text = [childNode.resourcePath lastPathComponent];
	
	BOOL hasChildren = [[childNode hasChildren] boolValue];
	[cell setAccessoryType:(hasChildren ? UITableViewCellAccessoryDisclosureIndicator : UITableViewCellAccessoryNone)];
	
	if (childNode.generation == nil)
	{
		[cell.uploadIndicator setHidden:NO];
		[cell.uploadIndicator startAnimating];
	}
	else
	{
		[cell.uploadIndicator setHidden:YES];
		[cell.uploadIndicator stopAnimating];
	}
	
	[cell.imageView setImage:nil];
	if ([childNode isFolder])
	{
		[cell.imageView setImage:[UIImage imageNamed:@"ic_folder"]];
		[cell.detailLabel setText:[childNode.modificationDate description]];
	}
	else
	{
		NSString *contentPath = childNode.contentPath;
		NSURL *contentURL = [NSURL fileURLWithPath:contentPath];
		UIDocumentInteractionController *docController = [UIDocumentInteractionController interactionControllerWithURL:contentURL];
		[cell.imageView setImage:[docController.icons objectAtIndex:0]];
		NSNumber *size = [childNode valueForKey:@"size"];
		NSString *sizeString = [self.byteSizeTransformer transformedValue:size];
		NSString *detailString = [sizeString stringByAppendingFormat:@", modified %@", childNode.modificationDate];
		[cell.detailLabel setText:detailString];
	}

	return cell;
}


#pragma mark UITableViewDelegate Methods

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath;
{
	U1Node *childNode = [self.resultsController objectAtIndexPath:indexPath];
	[self.delegate folderController:self didSelectNode:childNode];
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath;
{
	if (editingStyle != UITableViewCellEditingStyleDelete)
		return;
	
	[self deleteNodeAtIndexPath:indexPath];
}


- (void)refresh;
{
	[self loadInfoForNode];
}


#pragma mark Private Methods

- (void)loadInfoForNode;
{
	NSParameterAssert(self.resourcePath != nil);

	[self.filesClient fetchNodeWithResourcePath:self.resourcePath completionBlock:^(U1Node *theNode, NSFetchedResultsController *childrenResultsController, NSError *error) {
		
		self.node = (id)theNode;
		self.resultsController = childrenResultsController;
		[self.resultsController setDelegate:self];
		[self stopLoading];
		[self.tableView reloadData];
	}];
}

- (void)deleteNodeAtIndexPath:(NSIndexPath*)indexPath;
{
	U1Node *childNode = [self.resultsController objectAtIndexPath:indexPath];
	[self.filesClient deleteNode:childNode completionBlock:^(NSError *error) {
		if (error)
		{
			// TODO:
			return;
		}
	}];
}


#pragma mark NSFetchedResultsControllerDelegate Methods


- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller;
{
	[self.tableView beginUpdates];
}


- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
		   atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type;
{
	switch (type)
	{
		case NSFetchedResultsChangeInsert:
			[self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex]
						  withRowAnimation:UITableViewRowAnimationFade];
			break;
			
		case NSFetchedResultsChangeDelete:
			[self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex]
						  withRowAnimation:UITableViewRowAnimationFade];
			break;
	}
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
	   atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
	  newIndexPath:(NSIndexPath *)newIndexPath;
{
	
	UITableView *tableView = self.tableView;
	
	switch (type)
	{
			
		case NSFetchedResultsChangeInsert:
			[tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath]
							 withRowAnimation:UITableViewRowAnimationFade];
			break;
			
		case NSFetchedResultsChangeDelete:
			[tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
							 withRowAnimation:UITableViewRowAnimationFade];
			break;
			
		case NSFetchedResultsChangeUpdate:
			// TODO: 
			//            [self configureCell:[tableView cellForRowAtIndexPath:indexPath]
			//					atIndexPath:indexPath];
			[self.tableView reloadRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
								  withRowAnimation:UITableViewRowAnimationNone];
			break;
			
		case NSFetchedResultsChangeMove:
			[tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
							 withRowAnimation:UITableViewRowAnimationFade];
			[tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath]
							 withRowAnimation:UITableViewRowAnimationFade];
			break;
	}
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller;
{
	[self.tableView endUpdates];
}

@end
