//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1VolumesViewController.h"

#import "U1FilesService.h"


@interface U1VolumesViewController ()
@property (retain) NSArray *volumes;
@end


@implementation U1VolumesViewController

@synthesize loadingCell;
@synthesize delegate, filesService;
@synthesize volumes;

- (id)init;
{
	self = [super initWithNibName:@"U1VolumesViewController" bundle:nil];
	if (!self)
		return nil;
	self.title = NSLocalizedString(@"Home", @"");
	UIImage *logoImage = [UIImage imageNamed:@"logo_4_white"];
	UIImageView *logoView = [[UIImageView alloc] initWithImage:logoImage];
	[self.navigationItem setTitleView:logoView];
	[logoView release];
	return self;
}

- (void)dealloc;
{
	[loadingCell release];
	[filesService release];
	[volumes release];
	[super dealloc];
}

- (void)viewDidLoad;
{
	[super viewDidLoad];
	[self.filesService volumeInfoWithCompletionBlock:^(NSArray *volumeInfo, NSError *error) {
		dispatch_async(dispatch_get_main_queue(), ^(void) {
			self.volumes = volumeInfo;
			[self.tableView reloadData];
		});
	}];
}

- (void)viewDidUnload;
{
	[loadingCell release], loadingCell = nil;
	[super viewDidUnload];
}

- (void)viewDidAppear:(BOOL)animated;
{
	[super viewDidAppear:animated];
	[self.tableView deselectRowAtIndexPath:[self.tableView indexPathForSelectedRow] animated:YES];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section;
{
	if (self.volumes)
		return [self.volumes count];
	return 1; // Loading cell
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath;
{
	if (!self.volumes)
		return self.loadingCell;
	
	UITableViewCell *cell = [self.tableView dequeueReusableCellWithIdentifier:@"U1FolderCell"];
	if (!cell)
	{
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"U1FolderCell"] autorelease];
		[cell setAccessoryType:UITableViewCellAccessoryDisclosureIndicator];
		[cell setSelectionStyle:UITableViewCellSelectionStyleGray];
		[cell.imageView setImage:[UIImage imageNamed:@"ic_folder"]];
	}
	NSDictionary *volume = [self.volumes objectAtIndex:indexPath.row];
	NSString *volumePath = [volume objectForKey:@"path"];
	if ([@"~/.ubuntuone/Purchased from Ubuntu One" isEqualToString:volumePath])
		volumePath = @"Purchased Music";
	if ([volumePath hasPrefix:@"~/"])
		volumePath = [volumePath substringFromIndex:2];
	cell.textLabel.text = volumePath;
	
	return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath;
{
	NSDictionary *volume = [self.volumes objectAtIndex:indexPath.row];
	[self.delegate volumesController:self didSelectVolume:volume];
}

@end
